clear all
close all

%% Initialization 

% Load grid data
grid_data = load('gridData_large'); %Initierar data för beräkningar
load('gridStructure_large.mat'); %Inkluderar information om nätets struktur

% Extract per unit parameters
pu = PerUnit(grid_data); %Exempelvis effekterna i beräkningarna är i per-unit. Använd pu för att beräkna värden i exempelvis VA.

%% Generate forward backward sweep solver
% Om c-koden inte är kompilerad ska det göras här. Öppna -cc-filen och
% kompilera. Möjligt att flaggorna behöver tas bort beroende på
% operativsystem.

eval(['ForwardBackwardSweep = @(Z_ser, Y_shu, S_bus, U_bus, n_time) ', grid_structure.name, '(Z_ser, Y_shu, complex(S_bus), complex(U_bus), n_time);']); 

%% Evaluate forward backward sweep model

% Initialize data

timeline = 6552:(6552+7*24); %Antalet timmar in på året. Används för att simulera första veckan i oktober.

S_bus = grid_data.S_bus(:,timeline); %Hushållskonsumtion i slutförbrukarna. Utan elbil och solceller. Antaget effektfaktor=1. Tidsdiskretiseringen är en timme.
U_bus = grid_data.U_bus(:,timeline); %Initialgissning på spänningen i de olika bussarna för olika tider.

%S_bus = S_bus + j*S_bus

fprintf('Run FBS on %s for interval %d:%d... ', grid_structure.name, timeline(1),timeline(end));
tic;
[S_sim,U_sim,I_sim] = ForwardBackwardSweep(grid_structure.Z_ser,grid_structure.Y_shu,S_bus,U_bus, numel(timeline)); %Kör en beräkning på hushållskonsumtionsdata
ctime = toc; %Beräkningstid
fprintf('done.\n');

fprintf('CPU time: %f s, (%f s/sample) \n', ctime, ctime/numel(timeline));

%% Create adjecency matrix from edges info
grid_structure.matrix.Z_ser = grid_data.Z_ser;

grid_structure.matrix.Y_shu = grid_data.Y_shu;

grid_structure.matrix.edges = zeros(grid_structure.n_edges); %Matris som innehåller information om vilka noder som är sammankopplade med en kabel.
for e = 1:grid_structure.n_edges
    grid_structure.matrix.edges(grid_structure.edges(e,1), grid_structure.edges(e,2)) = 1;
    grid_structure.matrix.edges(grid_structure.edges(e,2), grid_structure.edges(e,1)) = 1;
end

%% Generate and plot graph

% Generate graph object
G = graph(grid_structure.matrix.edges);

% Plot graph where node color relates to voltage
plot(G,'NodeCData',min(abs(U_sim),[],2)*pu.phaseScale, 'LineWidth', 2, 'MarkerSize', 5, 'Layout','layered');
colormap winter
colorbar
title('Minimum voltage in each node');

householdsIndexes = [6,7,8,9,10,11,12,13,15,16,17,18,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35]; %De noder som är hushåll

%% Adding ev load
vehicle_data.battery_capacity = 100; % kWh
vehicle_data.efficiency = 0.95; % Batteriets verkningsgrad vid laddning.
vehicle_data.charging_power =5; % kW 

soc_constraints.soc_full =0.9; %Batteriets laddningsnivå när det är fulladdat.

ev_charging_periods.avg_trip_consumption = 20/vehicle_data.battery_capacity;      % Hur mycket laddningsnivå som används mellan varje laddningstillfälle.
ev_charging_periods.hours = [1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1; ... % Anger när fordonet är hemma under veckan. Här urkopplat och används vargdagar 8-17.
                             1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1; ...
                             1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1; ...
                             1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1; ...
                             1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1; ...
                             1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1; ...
                             1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1];                      
  
tic 
[S_load_week, SoC_trajectory_week, charging_periods_week] = GenerateEVChargingStrategy(vehicle_data, soc_constraints, ev_charging_periods); %Skapar laddningstrajektorier för fordonen under veckan.                        
toc

% Plot 

figure('Position', [1 1 1500 500]);
subplot(311);
stem(S_load_week);
ylabel('Laddeffekt [kW]')
xlim([1 24*7]);
xticks(0:6:24*7);
subplot(312);
plot(SoC_trajectory_week);
ylabel('SOC')
xlim([1 24*7]);
xticks(0:6:24*7);
subplot(313);
stem(charging_periods_week);
ylabel('Bil inkopplad?')
xlim([1 24*7]);
xticks(0:6:24*7);
xlabel('Tid [Timmar]')

S_EV = repmat(S_load_week,53,1);
S_EV = S_EV(49:49+365*24-1).'*pu.powerPhase; % Skalar S_load till per unit

vehicle_home = repmat(charging_periods_week, 52,1); %Skalar om laddningstrajektorian till samma storlek som hushållskonsumtionen.
vehicle_home = vehicle_home(49:49+365*24-1).';

%% Load PV data
num_pv_modules = 30; %Antal solcellsmoduler varje installation har
load('P_solar_2017_LG_40mod') % Total solcellsproduktion vid 40 moduler. [W]
P_pv_1mod = (P_simulation'./1000)/40; % Solcellsproduktion för en modul. [kW]

figure;
plot(num_pv_modules*P_pv_1mod)
title('Solproduktion')
xlabel('Tid [Timmar]')
ylabel('Effekt [kW]')

% Generate PV production 
S_PV = num_pv_modules*P_pv_1mod*pu.powerPhase; %Solcellsproduktion för anläggningen

%% Plot simulation w/wo EV and PV
EV_indexes = householdsIndexes; %Anger vilka hushåll som har elbilar. Kan ändras!!! Vektor anges i formatet [index_1 index_2 osv]
PV_indexes = householdsIndexes ; %Anger vilka hushåll som har solproduktion. Kan ändras!!! Vektor anges i formatet [index_1 index_2 osv]

S_EV_matrix = zeros(size(S_bus));
S_PV_matrix = zeros(size(S_bus));
S_EV_matrix(EV_indexes,:) = repmat(S_EV(timeline),length(EV_indexes),1); %Skapar matris med elbilsladdningseffekt. Noder och tid är dimensionerna.
S_PV_matrix(PV_indexes,:) = repmat(S_PV(timeline),length(PV_indexes),1); %Skapar matris med dolcellseffekt. Noder och tid är dimensionerna.

[S_simEV,U_simEV,I_simEV] = ForwardBackwardSweep(grid_structure.Z_ser,grid_structure.Y_shu,S_bus+S_EV_matrix,U_bus, numel(timeline)); %Beräkning med hushållskonsumtion och elbilsladdning.
[S_simPV,U_simPV,I_simPV] = ForwardBackwardSweep(grid_structure.Z_ser,grid_structure.Y_shu,S_bus-S_PV_matrix,U_bus, numel(timeline)); %Beräkning med hushållskonsumtion och solcellsproduktion.
[S_simEVPV,U_simEVPV,I_simEVPV] = ForwardBackwardSweep(grid_structure.Z_ser,grid_structure.Y_shu,S_bus-S_PV_matrix+S_EV_matrix,U_bus, numel(timeline)); %Beräkning med hushållskonsumtion, elbilsladdning, och solproduktion.

S_sim_maxTrafo = max(abs(S_sim(1,:)))*pu.S_base;
S_simEV_maxTrafo = max(abs(S_simEV(1,:)))*pu.S_base;
S_simPV_maxTrafo = max(abs(S_simPV(1,:)))*pu.S_base;
S_simEVPV_maxTrafo = max(abs(S_simEVPV(1,:)))*pu.S_base;

t=datetime(2020,1,1,0,0,0,'Format','eee yyyy-MM-dd HH:mm')+ hours(timeline); %Snyggare plotformat på x-axeln.

figure(4); %Exempelfigur
ax(1) = subplot(2,2,1);
plot(t,abs(U_sim(grid_structure.node_is_load,:))'*pu.U_sec_base/sqrt(3))
title('Nominal');
ylabel('Spänning [V]')
grid on
ax(2) = subplot(2,2,2);
plot(t,abs(U_simEV(grid_structure.node_is_load,:))'*pu.U_sec_base/sqrt(3))
title('EV');
grid on
ax(3) = subplot(2,2,3);
plot(t,abs(U_simPV(grid_structure.node_is_load,:))'*pu.U_sec_base/sqrt(3))
ylabel('Spänning [V]')
title('PV');
grid on
ax(4) = subplot(2,2,4);
plot(t,abs(U_simEVPV(grid_structure.node_is_load,:))'*pu.U_sec_base/sqrt(3))
title('EV and PV');
linkaxes(ax)
xlim([t(1) t(end)])
grid on

%Exempelfigur när hushållsförbrukning och PV aktiverade
% figure(6)
% plot(t,abs(U_simPV(grid_structure.node_is_load,:))'*pu.U_sec_base/sqrt(3))
% ylabel('Spänning [V]')
% title('PV');
% grid on

% Plot graph where node color relates to maximum voltage when PV is added
% figure(5)
% plot(G,'NodeCData',max(abs(U_simPV),[],2)*pu.phaseScale, 'LineWidth', 2, 'MarkerSize', 5, 'Layout','layered');
% colormap winter
% colorbar
% title('Maximum voltage in each node');




